/* Copyright 2000 Red Hat, Inc.
 *
 * Than Ngo <than@redhat.de>
 *
 * This software may be freely redistributed under the terms of the GNU
 * public license.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


#include <qdir.h>
#include <qfile.h>
#include <qfileinfo.h>
#include <qstringlist.h>
#include <qtextstream.h>
#include <iostream>
#include <fstream>
#include <strstream>
#include <stdlib.h>
#include <unistd.h> // unlink, rename
#include <sys/types.h>
#include <sys/stat.h>

#include "generaldata.h"
#include <qregexp.h>

#define ACTIVE_ISDN "/etc/sysconfig/activeisdn"
#define ISDN_CARD "/etc/sysconfig/isdncard"
#define PROVIDER_DIR "/etc/sysconfig/provider"
#define PAP_SECRETS "/etc/ppp/pap-secrets"
#define CHAP_SECRETS "/etc/ppp/chap-secrets"
#define OLDCONFMODUL "/etc/conf.modules" 
#define CONFMODUL "/etc/modules.conf"
#define NETWORK "/etc/sysconfig/network"
#define PROC_PCI "/proc/bus/pci/devices"

const char *GeneralData::encap_items[] = { "Sync PPP", "Raw IP", NULL };
const char *GeneralData::auth_items[] = { "PAP", "CHAP", "PAP+CHAP", "NONE", NULL };

GeneralData ISDNData;

GeneralData::GeneralData()
{
  providerdata.setAutoDelete(TRUE);
}

void GeneralData::save()
{
  writeisdncardfile();
  writeactiveisdnfile();
  writeconfigfile();
  writesecretfile(AUTH_PAP);
  writesecretfile(AUTH_CHAP);
  writenetworkfile();
}

void GeneralData::writenetworkfile()
{
  QString gateway;
  QString gatewaydev;
	uint counter = 0;
  bool add_gateway = FALSE;
					
  for (ProviderData *pd = providerdata.first(); pd != 0; pd = providerdata.next()) {
    // provider is configured with dial on demand
    if (pd->dialmode == "auto") {
      add_gateway = TRUE;
      gateway = pd->remoteip;
      // we allow only one provider configured with dial on demand
      // we take the fisrt provider from the list
      // we find out the isdn interface
      if (pd->encapsulation == "syncppp")
        gatewaydev=QString("ippp") + QString::number(counter);
      else
        gatewaydev=QString("isdn") + QString::number(counter);

			break;
    }
    counter++;
  }

  // don't add new gateway and gatewaydev when dialmode=manual
  if ( !add_gateway )
    return;

  // syncppp with dynamic IP and dial on demand
  // it's required to set gateway=0.0.0.0
  if (gateway.isEmpty())
    gateway="0.0.0.0";

  QFile file(NETWORK);

  if (!file.open(IO_ReadOnly)) {
    cerr << NETWORK << " is not available!" << endl;
    return;
  }

  backupfile(NETWORK);

  QStrList nw;
  QTextStream ts(&file);
  QString s;
	
  // read network file and replace gateway, gatewaydev
  while (!ts.eof()) {
    s = ts.readLine();
    if (!s.contains("GATEWAY=") && !s.contains("GATEWAYDEV="))
      nw.append(s);
	}
  nw.append(QString("GATEWAY=") + gateway);
  nw.append(QString("GATEWAYDEV=") + gatewaydev);
  file.close();
  
  // write network file
  if (file.open(IO_WriteOnly)) {
    QTextStream ts(&file);
    for (unsigned int i = 0; i < nw.count(); i++)
      ts << nw.at(i) << endl;
  }

	file.close();
}

void GeneralData::backupfile(QString s)
{
  QFile file(s);
  if (file.exists())
   {
    QString n=s+".orig";
    ifstream i(s);
    if(!i) return;
    ofstream o(n);
    if(!o) { i.close(); return; }
    while(!i.eof())
     {
      char c;
      i.get(c);
      if(i.eof()) break;
      o << c;
     }
    i.close();
    o.close();
    struct stat os;
    if(!stat(s,&os))
     {
      chown(n,os.st_uid,os.st_gid);
      chmod(n,os.st_mode);
     }
  }
}

void GeneralData::movefile(QString s)
{
  QFile file(s);

  if (file.exists()) {
    unlink(s+".orig");
    rename(s,s+".orig");
  }
}

// write providerconfiguration
void GeneralData::writeconfigfile()
{
  QFile file;
  QDir dir(PROVIDER_DIR);
  QString configfile = QString(PROVIDER_DIR) + QString("/conf-");

  if (!dir.exists()) {
    if (!dir.mkdir(PROVIDER_DIR)) {
      qWarning("Could not create directory");
      return;
    }
  }

  ProviderData *pd;

  // backup configfile if no privider avaiable
  if (providerdata.isEmpty()) {
    for (uint i = 0; i < configfilenamelist.count(); i++) {
      movefile(configfile + configfilenamelist.at(i));
		}
  }
  // move configfile
  else {
    for (uint i = 0; i < configfilenamelist.count(); i++) {
      for (pd = providerdata.first(); pd != 0; pd = providerdata.next()) {
        if (pd->providername != configfilenamelist.at(i))
          movefile(configfile + configfilenamelist.at(i));
      }
    }
  }
  
  for (pd = providerdata.first(); pd != 0; pd = providerdata.next()) {
    backupfile(configfile + pd->providername);
    file.setName(configfile + pd->providername);
    if (file.open(IO_WriteOnly)) {
      QTextStream ts(&file);
      ts << "PROVIDER=\"" << pd->providername << "\"" << endl;
      ts << "USER=\"" << pd->loginname << "\"" << endl;
      ts << "PASSWORD=\"" << pd->password << "\"" << endl;
      ts << "ENCAP=\"" << pd->encapsulation << "\"" << endl;
      ts << "DIALMODE=\"" << pd->dialmode << "\"" << endl;
      ts << "SECURE=\"" << pd->secure << "\"" << endl;
      ts << "MSN=\"" << pd->msn << "\"" << endl;
      ts << "PHONE_IN=\"" << pd->phone_in << "\"" << endl;
      // ts << "PREFIX=\"" << pd->prefix << "\"" << endl;
      ts << "AREACODE=\"" << pd->areacode << "\"" << endl;
      ts << "PHONE_OUT=\"" << pd->phonenumber << "\"" << endl;
      // ts << "CITY=\"" << pd->city << "\"" << endl;
      // ts << "COUNTRY=\"" << pd->country << "\"" << endl;
      ts << "BUNDLING=\"" << pd->bundling << "\"" << endl;
      ts << "LOCAL_IP=\"" << pd->localip << "\"" << endl;
      ts << "REMOTE_IP=\"" << pd->remoteip << "\"" << endl;
      // ts << "NETWORK=\"" << pd->network << "\"" << endl;
      // ts << "BROADCAST=\"" << pd->broadcast << "\"" << endl;
      ts << "HUPTIMEOUT=\"" << pd->huptimeout << "\"" << endl;
      ts << "DNS=\"" << pd->dns << "\"" << endl;
      // ts << "HOSTNAME=\"" << pd->hostname << "\"" << endl;
      ts << "DOMAIN=\"" << pd->domain << "\"" << endl;
      // ts << "L2_PROT=\"" << pd->l2_prot << "\"" << endl;
      // ts << "L3_PROT=\"" << pd->l3_prot << "\"" << endl;
      ts << "CALLBACK=\"" << pd->callback << "\""  << endl;
      ts << "CHARGEHUP=\"" << pd->chargehup << "\""  << endl;
      ts << "CHARGEINT=\"" << pd->chargeint << "\""  << endl;
      ts << "CBHUP=\"" << pd->cbhup << "\""  << endl;
      ts << "CBDELAY=\"" << pd->cbdelay << "\""  << endl;
      ts << "DIALMAX=\"" << pd->dialmax << "\""  << endl;
      ts << "AUTH=\"" << pd->auth << "\"" << endl;
      ts << "IHUP=\"" << pd->ihup << "\""  << endl;
      // ts << "IP_FORWARD=\"" << pd->ip_forward << "\""  << endl;
      // ts << "SLAVE_DELAY=\"" << pd->slave_delay << "\""  << endl;
      // ts << "SLAVE_TRIGGER=\"" << pd->slave_trigger << "\""  << endl;
      // ts << "SLAVE_DIALMODE=\"" << pd->slave_dialmode << "\""  << endl;
      // ts << "SLAVE_MSN=\"" << pd->slave_msn << "\""  << endl;
      // ts << "SLAVE_PHONE_IN=\"" << pd->slave_phone_in << "\"" << endl;
      // ts << "SLAVE_PHONE_OUT=\"" << pd->slave_phonenumber << "\"" << endl;
      // ts << "SLAVE_HUPTIMEOUT=\"" << pd->slave_huptimeout << "\"" << endl;
      // ts << "SLAVE_CALLBACK=\"" << pd->slave_callback << "\""  << endl;
      // ts << "SLAVE_CHARGEHUP=\"" << pd->slave_chargehup << "\""  << endl;
      // ts << "SLAVE_CHARGEINT=\"" << pd->slave_chargeint << "\""  << endl;
      // ts << "SLAVE_CBHUP=\"" << pd->slave_cbhup << "\""  << endl;
      // ts << "SLAVE_CBDELAY=\"" << pd->slave_cbdelay << "\""  << endl;
      // ts << "SLAVE_DIALMAX=\"" << pd->slave_dialmax << "\""  << endl;
      // ts << "SLAVE_IHUP=\"" << pd->slave_ihup << "\""  << endl;
      ts << "DELDEFAULTROUTE=\"" << pd->deldefaultroute << "\"" << endl;
      for (unsigned int i = 0; i < pd->unknown.count(); i++)
        ts << pd->unknown.at(i) << endl;
      
      file.close();
      if (chmod(file.name().ascii(), S_IRUSR | S_IWUSR) < 0)
        cerr << "cannot chmod" << file.name() << endl;
    }
  }
}

// write configurations in CHAP and PAP-secret files
void GeneralData::writesecretfile(uint _auth)
{
  QFile file;
  QStrList list;
  QStrList oldlist;
  QString str;
  
  switch (_auth) {
  case AUTH_PAP:
    file.setName(PAP_SECRETS);
    str = "+pap";
    break;
  case AUTH_CHAP:
    file.setName(CHAP_SECRETS);
    str = "+chap";
    break;
  }

  for (ProviderData *pd = providerdata.first(); pd != 0;
       pd = providerdata.next()) {
    if (pd->auth.contains(str))
      list.append(QString("\"") + pd->loginname + QString("\"\t") + QString("*") +
                  QString("\t\"") + pd->password + QString("\""));
  }

  if (file.size() != 0)
    backupfile(file.name());

  // read secret file
  if (file.open(IO_ReadOnly)) {
    QTextStream ts(&file);
    QString s;
    while (!ts.eof()) {
      s = ts.readLine();
      // ignore all lines in this section
      if (s ==
          "####### isdn-config will overwrite this part!!! (begin)##########") {
        while (ts.readLine() !=
               "####### isdn-config will overwrite this part!!! (end)############");
        continue;
      }
      oldlist.append(s);
    }
    file.close();
  }
  // write secret file
  if (file.open(IO_WriteOnly)) {
    QTextStream ts(&file);
    // write old
    if (!oldlist.isEmpty()) {
      for (uint i = 0; i < oldlist.count(); i++)
        ts << oldlist.at(i) << endl;
    }
    // write new
    if (!list.isEmpty()) {
      ts <<
        "####### isdn-config will overwrite this part!!! (begin)##########"
         << endl;
      for (uint i = 0; i < list.count(); i++)
        ts << list.at(i) << endl;
      ts <<
        "####### isdn-config will overwrite this part!!! (end)############"
         << endl;
    }
    
    file.close();
    
    if (chmod(file.name().ascii(), S_IRUSR | S_IWUSR) < 0)
      cerr << "cannot chmod" << file.name() << endl;
  }
}

// write activeisdn file
void GeneralData::writeactiveisdnfile()
{
  QFile file(ACTIVE_ISDN);
  backupfile(ACTIVE_ISDN);

  if (file.open(IO_WriteOnly)) {	// file opened successfully
    QTextStream ts(&file);
    for (ProviderData * pd = providerdata.first(); pd != 0;
         pd = providerdata.next()) {
      if (pd->isproviderenabled)
        ts << pd->providername << endl;
    }
    file.close();
  }
}

// write isdncard file
void GeneralData::writeisdncardfile()
{
  QFile file(ISDN_CARD);
  backupfile(ISDN_CARD);

  if (!resourcedata.isenabled)
    file.remove(); // isdncard is disabled
  else {
    if (file.open(IO_WriteOnly)) { // file opened successfully
      QTextStream ts(&file);
      // write resourcedata
      ts << "NAME=\"" << resourcedata.isdncardname << "\"" << endl;
      ts << "MODULE=\"" << resourcedata.module << "\"" << endl;
      if ( resourcedata.firmware.length() > 0)
        ts << "FIRMWARE=\"" << resourcedata.firmware << "\"" << endl;
      ts << "RESOURCES=\"";
      if (resourcedata.type > 0)
        ts << "type=" << resourcedata.type << " ";
      ts << "protocol=" << resourcedata.protocol;
      if (resourcedata.irq != 0)
        ts << " irq=" << resourcedata.irq;
      if (resourcedata.id.length() > 0)
        ts << " id=" << resourcedata.id;
      if (resourcedata.io.length() > 0) {
        switch (resourcedata.type) {
        case 4:  // Creatix/Teles PnP
        case 19: // Compaq ISDN S0 ISA card
        case 24: // Dr. Neuhaus Niccy PnP
          ts << " io0=" << resourcedata.io;
          break;
        default:
          ts << " io=" << resourcedata.io;
          break;
        }
      }
      if (resourcedata.io1.length() > 0)
        ts << " io1=" << resourcedata.io1;
      if (resourcedata.io2.length() > 0)
        ts << " io2=" << resourcedata.io2;
      if (resourcedata.mem.length() > 0)
        ts << " mem=" << resourcedata.mem;
      
      ts << "\"" << endl;
      file.close();
    }
  }
}

void GeneralData::load()
{
  // first clear providerdata
  ISDNData.providerdata.clear();
  configfilenamelist.clear();

  // Load isdncard data
  loadisdncardData();

  // check pci card on system
  detect();

  // Mark active ISDN configurations
  QFile file;
  QStrList activeproviderlist;

  file.setName(ACTIVE_ISDN);
  if (file.open(IO_ReadOnly)) {
    QTextStream ts(&file);
    QString s;
    while (!ts.eof()) {
      s = ts.readLine();
      activeproviderlist.append(s);
    }
    file.close();
  }
  
  configfilenamelist = activeproviderlist;

  // mark all ISDN configurations under /etc/sysconfig/provider
  QDir dir;
  dir.setFilter(QDir::Files | QDir::Hidden | QDir::NoSymLinks);
  dir.setNameFilter("conf-*");
  
  // go to the provider directory and check if we can read it
  if (!dir.cd(PROVIDER_DIR) || !dir.isReadable()) {
    qWarning("\'PROVIDER_DIR\' does not exist or is unreadable");
    return;
  }

  for (unsigned int i = 0; i < dir.count(); i++) {
    // all config files with extension ignored
    if (QFileInfo(dir[i].ascii()).extension(true).isEmpty())
      if (!activeproviderlist.contains(dir[i].replace(QRegExp("conf-"), ""))) {
	// add only inactiv provider
	configfilenamelist.append(dir[i].replace(QRegExp("conf-"), ""));
      }
  }
	
  for (unsigned int i = 0; i < configfilenamelist.count(); i++) {
    loadProviderData((QString) dir.path().ascii() + "/conf-" +
		     configfilenamelist.at(i));
  }

  for (unsigned int i = 0; i < activeproviderlist.count(); i++) {
    for (ProviderData *pd = providerdata.first(); pd != 0; pd = providerdata.next()) {
      if (pd->providername == activeproviderlist.at(i)) {
	pd->isproviderenabled = true;
	break;
      }
    }
  }
}

void GeneralData::loadisdncardData()
{
  uint irq = 0;
  QString io("");
  QString io1("");
  QString io2("");
  QString mem("");

  // Load isdn configuration
  QFile file(ISDN_CARD);
	
  if (file.open(IO_ReadOnly)) {
    QTextStream ts(&file);
    QString s;
    resourcedata.isenabled = true; // isdncard is configured
  
    while (!ts.eof()) {
      s = ts.readLine(); // line of text excluding '\n'
      // parse data
      if (s.at(1) == '#');  // ignore lines beginning with '#'
      else if (s.contains("NAME="))
        resourcedata.isdncardname = getValue(s);
      else if (s.contains("MODULE="))
        resourcedata.module = getValue(s);
      else if (s.contains("FIRMWARE="))
        resourcedata.firmware = getValue(s);
      else if (s.contains("RESOURCES=")) {
        QStringList list = QStringList::split(' ', getValue(s));
        QStringList::Iterator s = list.begin();
        for (s = list.begin(); s != list.end(); ++s) {
          if ((*s).contains("type="))
            resourcedata.type = getValue(*s).toUInt();
          else if ((*s).contains("protocol="))
            resourcedata.protocol = getValue(*s).toUInt();
          else if ((*s).contains("irq="))
            irq = getValue(*s).toUInt();
          else if ((*s).contains("id="))
            resourcedata.id = getValue(*s);
          else if ((*s).contains("io=") || (*s).contains("io0="))
            io = getValue(*s);
          else if ((*s).contains("io1="))
            io1 = getValue(*s);
          else if ((*s).contains("io2="))
            io2 = getValue(*s);
          else if ((*s).contains("mem="))
            mem = getValue(*s);
        }
      }
    }
    file.close();
    resourcedata.irq = irq;
    resourcedata.io = io;
    resourcedata.io1 = io1;
    resourcedata.io2 = io2;
    resourcedata.mem = mem;
  } else {
    if (loadconfmodulfile())
      resourcedata.isenabled = true;
    else
      resourcedata.isenabled = false;
  }
}

bool GeneralData::loadProviderData(QString pfile)
{
  QFile f(pfile);

  if (f.open(IO_ReadOnly)) {
    ProviderData *pd = new ProviderData;

    QTextStream ts(&f);
    QString s;
    while (!ts.eof()) {
      s = ts.readLine();
      if (s.at(1) == '#')
        pd->unknown.append(s); // save files beginning with '#'
      else if (s.contains("PROVIDER="))
        pd->providername = getValue(s);
      else if (s.contains("USER="))
        pd->loginname = getValue(s);
      else if (s.contains("PASSWORD="))
        pd->password = getValue(s);
      else if (s.contains("ENCAP="))
        pd->encapsulation = getValue(s);
      else if (s.contains("DIALMODE="))
        pd->dialmode = getValue(s);
      else if(s.contains("SECURE="))
        pd->secure = getValue(s);
      else if (s.contains("MSN="))
        pd->msn = getValue(s);
      else if (s.contains("PHONE_IN="))
        pd->phone_in = getValue(s);
      //else if (s.contains("PREFIX="))
      //  pd->prefix = getValue(s);
      else if (s.contains("AREACODE="))
        pd->areacode = getValue(s);
      else if (s.contains("PHONE_OUT="))
        pd->phonenumber = getValue(s);
      else if (s.contains("CITY="))
        pd->city = getValue(s);
      else if (s.contains("COUNTRY="))
        pd->country = getValue(s);
      else if (s.contains("BUNDLING="))
        pd->bundling = getValue(s);
      else if (s.contains("LOCAL_IP=")) {
        pd->localip = getValue(s);
        if (pd->localip.length() > 0) {
          pd->islocalipenabled = TRUE;
        }
      }
      else if (s.contains("REMOTE_IP=")) {
        pd->remoteip = getValue(s);
        if (pd->remoteip.length() > 0)
          pd->isremoteipenabled = TRUE;
      }
      else if (s.contains("NETWORK="))
        pd->network = getValue(s);
      else if (s.contains("BROADCAST="))
        pd->broadcast = getValue(s);
      else if (s.contains("HUPTIMEOUT="))
        pd->huptimeout = getValue(s).toUInt();
      else if (s.contains("DNS="))
        pd->dns = getValue(s);
      else if (s.contains("HOSTNAME="))
        pd->hostname = getValue(s);
      else if (s.contains("DOMAIN="))
        pd->domain = getValue(s);
      else if (s.contains("L2_PROT="))
        pd->l2_prot = getValue(s);
      else if (s.contains("L3_PROT="))
        pd->l3_prot = getValue(s);
      else if (s.contains("CALLBACK="))
        pd->callback = getValue(s);
      else if (s.contains("CHARGEHUP"))
        pd->chargehup = getValue(s);
      else if (s.contains("CHARGEINT="))
        pd->chargeint = getValue(s);
      else if (s.contains("CBHUP="))
        pd->cbhup = getValue(s);
      else if (s.contains("CBDELAY="))
        pd->cbdelay = getValue(s);
      else if (s.contains("DIALMAX="))
        pd->dialmax = getValue(s);
      else if (s.contains("AUTH"))
        pd->auth = getValue(s);
      else if (s.contains("IHUP="))
        pd->ihup = getValue(s);
      else if (s.contains("IP_FORWARD="))
        pd->ip_forward = getValue(s);
      else if (s.contains("DELDEFAULTROUTE"))
        pd->deldefaultroute = getValue(s);
      else
        pd->unknown.append(s);
    }
    f.close();
    providerdata.append(pd);
    return TRUE;
  }
  return FALSE;
}

bool GeneralData::loadconfmodulfile()
{
  QFile file(CONFMODUL);
	
  if (!file.open(IO_ReadOnly))
    return false;
  
  QTextStream ts(&file);
  QString s;
  uint irq = 0;
  QString io("");
  QString io1("");
  QString io2("");
  QString mem("");
  
  while (!ts.eof()) {
    s = ts.readLine();
    // ignore lines beginning with '#'
    if (s.stripWhiteSpace().left(1) == "#");
    else if (s.contains("options hisax")) {
      resourcedata.module = "hisax";
      resourcedata.id = "HiSax";
      QStringList list = QStringList::split(' ', s);
      QStringList::Iterator s = list.begin();
			
      for (s = list.begin(); s != list.end(); ++s) {
        if ((*s).contains("type="))
          resourcedata.type = getValue(*s).toUInt();
        else if ((*s).contains("protocol="))
          resourcedata.protocol = getValue(*s).toUInt();
        else if ((*s).contains("irq="))
          irq = getValue(*s).toUInt();
        else if ((*s).contains("id="))
          resourcedata.id = getValue(*s);
        else if ((*s).contains("io=") || (*s).contains("io0="))
          io = getValue(*s);
        else if ((*s).contains("io1="))
          io1 = getValue(*s);
        else if ((*s).contains("io2="))
          io2 = getValue(*s);
        else if ((*s).contains("mem="))
          mem = getValue(*s);
      }
			
      resourcedata.isdncardname = getisdncardname(resourcedata.type);
      resourcedata.irq = irq;
      resourcedata.io = io;
      resourcedata.io1 = io1;
      resourcedata.io2 = io2;
      resourcedata.mem = mem;

      file.close();
      writeisdncardfile();
      return true;
    }
  }
	
  file.close();

	return false;
}

QString GeneralData::getisdncardname(uint mytype)
{
  for (register int i = 0; ResourceData::isdntype[i] != NULL; i += ISDNTYPE_ITEMS) {
    if (QString(ResourceData::isdntype[i + 1]).toUInt() == mytype)
      return QString(ResourceData::isdntype[i]);
  }
  return QString("Unknown");
}

QString GeneralData::getValue(QString str)
{
  QString value;
  
  if (str.find('=') < 0)
    return QString("");
  
  // move all before, and '='
  str.remove(0, str.find('=') + 1);
  
  // strip white spaces
  str = str.stripWhiteSpace();

  // strip beginning and ending '"'
  if (str.find('"') == 0)
    str.remove(0, 1);
  if (str.findRev('"') == (int) str.length() - 1)
    str.remove(str.length() - 1, 1);
  
  // strip white spaces
  value = str.stripWhiteSpace();

  return value;
}

void GeneralData::detect()
{
  for ( register int i = 0; ResourceData::isdntype[i] != NULL; i += ISDNTYPE_ITEMS ) {
    if ( QString(ResourceData::isdntype[i+7]).length() > 0 )
    {
      // detect the PCI ISDN card on machine
      if ( scan_pci(ResourceData::isdntype[i+7]) )
      {
        if (resourcedata.isenabled && (ResourceData::isdntype[i] == resourcedata.isdncardname))
          break; // exit, this card was already configured. 

        // so the new isdn card was found, we configure it now
        resourcedata.isenabled = true;
        resourcedata.isdncardname = ResourceData::isdntype[i];
        resourcedata.firmware = ResourceData::isdntype[i+8];
        resourcedata.type = QString(ResourceData::isdntype[i+1]).toUInt();
        resourcedata.module = ResourceData::isdntype[i+9];
        if(resourcedata.module.compare("hisax"))
	  resourcedata.id="HiSax";
	else
	  resourcedata.id="";
        resourcedata.protocol = 2;
        resourcedata.irq = 0;
        resourcedata.io = "";
        resourcedata.io1 = "";
        resourcedata.io2 = "";
        resourcedata.mem = "";

        // create isdncard config file
        writeisdncardfile();
        break;
      }
    }
  }
}

bool GeneralData::scan_pci(QString pciid)
{
  ifstream file(PROC_PCI);

  if (!file) {
    cerr << "cannot open" << PROC_PCI << endl;
    return false;
  }

  strstream strstr;
  strstr << file.rdbuf();

  file.close();

  QString s((char *)strstr.str());

  return(s.find(pciid) < 0 ? false : true);
}


bool GeneralData::verify()
{
  return true;
}

bool GeneralData::isproviderexisted(QString pn)
{
  for (uint i = 0; i < providerdata.count(); i++)
    if (providerdata.at(i)->providername == pn)
      return true;
  return false;
}

QString GeneralData::getphonenumber(int _index)
{
  return providerdata.at(_index)->phonenumber;
}

GeneralData::~GeneralData()
{
}
